@@ -2,18 +2,21 @@
 **  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-filter.h,v 1.45 2007/03/07 15:50:09 msk Exp $
+**  $Id: dkim-filter.h,v 1.65 2007/05/23 18:16:02 msk Exp $
 */
 
 #ifndef _DKIM_FILTER_H_
 #define _DKIM_FILTER_H_
 
 #ifndef lint
-static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.45 2007/03/07 15:50:09 msk Exp $";
+static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.65 2007/05/23 18:16:02 msk Exp $";
 #endif /* !lint */
 
 #define	DKIMF_PRODUCT	"Sendmail DKIM Filter"
-#define	DKIMF_VERSION	"0.6.1"
+#define	DKIMF_VERSION	"1.0.0"
+
+/* libsm */
+#include <sm/gen.h>
 
 /* libdkim */
 #include <dkim.h>
@@ -28,10 +31,8 @@ static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.45 2007/03/07 15:50
 
 /* defaults, limits, etc. */
 #define	BUFRSZ		256
-#if _FFR_SET_DNS_CALLBACK
 #define	CBINTERVAL	3
-#endif /* _FFR_SET_DNS_CALLBACK */
-#define CMDLINEOPTS	"a:Ab:c:C:d:DfFhi:I:k:Klm:M:op:P:qrRs:S:T:u:U:v:Vx:"
+#define CMDLINEOPTS	"a:Ab:c:C:d:DfFhi:I:k:Klm:M:o:p:P:qrRs:S:T:u:U:Vx:"
 #define	DEFTIMEOUT	5
 #define	HOSTUNKNOWN	"unknown-host"
 #define	JOBIDUNKNOWN	"(unknown-jobid)"
@@ -66,6 +67,7 @@ struct Peer
 };
 
 /* externs */
+extern bool dolog;
 extern char *progname;
 
 #endif /* _DKIM_FILTER_H_ */
