@@ -2,11 +2,11 @@
 **  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: util.c,v 1.14 2007/03/01 22:32:08 msk Exp $
+**  $Id: util.c,v 1.34 2007/05/17 16:13:51 msk Exp $
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.14 2007/03/01 22:32:08 msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.34 2007/05/17 16:13:51 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -26,41 +26,66 @@ static char util_c_id[] = "@(#)$Id: util.c,v 1.14 2007/03/01 22:32:08 msk Exp $"
 #include <stdio.h>
 #include <netdb.h>
 #include <fcntl.h>
+#include <unistd.h>
 
 /* libsm includes */
 #include <sm/gen.h>
 #include <sm/string.h>
 
 #if POPAUTH
+/* system includes */
+# include <pthread.h>
+
 /* libdb includes */
-#include <db.h>
+# include <db.h>
 #endif /* POPAUTH */
 
 /* dkim-filter includes */
 #include "dkim-filter.h"
 #include "util.h"
 
-static char *ffrlist[] =
+/* globals */
+#if POPAUTH
+static pthread_mutex_t pop_lock;
+#endif /* POPAUTH */
+
+static char *optlist[] =
 {
+#if DEBUG
+	"DEBUG",
+#endif /* DEBUG */
+
+#if POPAUTH
+	"POPAUTH",
+#endif /* POPAUTH */
+
+#if VERIFY_DOMAINKEYS
+	"VERIFY_DOMAINKEYS",
+#endif /* VERIFY_DOMAINKEYS */
+
 #if _FFR_ALLMAN_SSP_02
 	"_FFR_ALLMAN_SSP_02",
 #endif /* _FFR_ALLMAN_SSP_02 */
 
+#if _FFR_ANTICIPATE_SENDMAIL_MUNGE
+	"_FFR_ANTICIPATE_SENDMAIL_MUNGE",
+#endif /* _FFR_ANTICIPATE_SENDMAIL_MUNGE */
+
+#if _FFR_CAPTURE_UNKNOWN_ERRORS
+	"_FFR_CAPTURE_UNKNOWN_ERRORS",
+#endif /* _FFR_CAPTURE_UNKNOWN_ERRORS */
+
 #if _FFR_FLUSH_HEADERS
 	"_FFR_FLUSH_HEADERS",
 #endif /* _FFR_FLUSH_HEADERS */
 
-#if _FFR_LOG_SSL_ERRORS
-	"_FFR_LOG_SSL_ERRORS",
-#endif /* _FFR_LOG_SSL_ERRORS */
+#if _FFR_HASH_BUFFERING
+	"_FFR_HASH_BUFFERING",
+#endif /* _FFR_HASH_BUFFERING */
 
-#if _FFR_MULTIPLE_KEYS
-	"_FFR_MULTIPLE_KEYS",
-#endif /* _FFR_MULTIPLE_KEYS */
-
-#if _FFR_REPORTINFO
-	"_FFR_REPORTINFO",
-#endif /* _FFR_REPORTINFO */
+#if _FFR_QUERY_CACHE
+	"_FFR_QUERY_CACHE",
+#endif /* _FFR_QUERY_CACHE */
 
 #if _FFR_REQUIRED_HEADERS
 	"_FFR_REQUIRED_HEADERS",
@@ -74,19 +99,23 @@ static char *ffrlist[] =
 	"_FFR_SELECT_SIGN_HEADERS",
 #endif /* _FFR_SELECT_SIGN_HEADERS */
 
-#if _FFR_SET_DNS_CALLBACK
-	"_FFR_SET_DNS_CALLBACK",
-#endif /* _FFR_SET_DNS_CALLBACK */
-
 #if _FFR_SET_REPLY
 	"_FFR_SET_REPLY",
 #endif /* _FFR_SET_REPLY */
 
+#if _FFR_STATS
+	"_FFR_STATS",
+#endif /* _FFR_STATS */
+
+#if _FFR_VBR
+	"_FFR_VBR",
+#endif /* _FFR_VBR */
+
 	NULL
 };
 
 /*
-**  DKIMF_FFRLIST -- print active FFRs
+**  DKIMF_OPTLIST -- print active FFRs
 **
 **  Parameters:
 **  	where -- where to write the list
@@ -96,22 +125,22 @@ static char *ffrlist[] =
 */
 
 void
-dkimf_ffrlist(FILE *where)
+dkimf_optlist(FILE *where)
 {
 	bool first = TRUE;
 	int c;
 
 	assert(where != NULL);
 
-	for (c = 0; ffrlist[c] != NULL; c++)
+	for (c = 0; optlist[c] != NULL; c++)
 	{
 		if (first)
 		{
-			fprintf(where, "\tActive FFRs:\n");
+			fprintf(where, "\tActive code options:\n");
 			first = FALSE;
 		}
 
-		fprintf(where, "\t\t%s\n", ffrlist[c]);
+		fprintf(where, "\t\t%s\n", optlist[c]);
 	}
 }
 
@@ -372,6 +401,22 @@ dkimf_checkip(Peer list, struct sockaddr *ip)
 }
 
 #if POPAUTH
+/*
+**  DKIMF_INITPOPAUTH -- initialize POPAUTH stuff
+**
+**  Parameters:
+**  	None.
+**
+**  Return value:
+**  	0 on success, an error code on failure.  See pthread_mutex_init().
+*/
+
+int
+dkimf_initpopauth(void)
+{
+	return pthread_mutex_init(&pop_lock, NULL);
+}
+
 /*
 **  DKIMF_CHECKPOPAUTH -- check a POP before SMTP database for client
 **                      authentication
@@ -425,6 +470,11 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 	q.data = ipbuf;
 	q.size = strlen(q.data);
 
+	/* we don't care about the value for now */
+# if DB_VERSION_MAJOR > 2
+	d.flags = DB_DBT_USERMEM|DB_DBT_PARTIAL;
+# endif /* DB_VERSION_MAJOR > 2 */
+
 	/* establish read-lock */
 	fd = -1;
 # if DB_VERSION_MAJOR >= 2
@@ -434,12 +484,16 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 	status = 0;
 	fd = db->fd(db);
 # endif /* DB_VERSION_MAJOR >= 2 */
+
+	/* XXX -- allow multiple readers? */
+	(void) pthread_mutex_lock(&pop_lock);
+
 	if (status == 0 && fd != -1)
 	{
 # ifdef LOCK_SH
 		status = flock(fd, LOCK_SH);
 
-		if (status != 0)
+		if (status != 0 && dolog)
 		{
 			syslog(LOG_WARNING, "flock(LOCK_SH): %s",
 			       strerror(errno));
@@ -448,13 +502,13 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 		struct flock l;
 
 		l.l_start = 0;
-		l.l_end = 0;
+		l.l_len = 0;
 		l.l_type = F_RDLCK;
 		l.l_whence = SEEK_SET;
 
 		status = fcntl(fd, F_SETLKW, &l);
 
-		if (status != 0)
+		if (status != 0 && dolog)
 		{
 			syslog(LOG_WARNING, "fcntl(F_RDLCK): %s",
 			       strerror(errno));
@@ -468,7 +522,13 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 	status = db->get(db, NULL, &q, &d, 0);
 # endif /* DB_VERSION_MAJOR */
 	if (status == 0)
+	{
 		ret = TRUE;
+	}
+	else if (status != DB_NOTFOUND && dolog)
+	{
+		syslog(LOG_ERR, "db->get(): %s", db_strerror(status));
+	}
 
 	/* surrender read-lock */
 	if (fd != -1)
@@ -476,7 +536,7 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 # ifdef LOCK_SH
 		status = flock(fd, LOCK_UN);
 
-		if (status != 0)
+		if (status != 0 && dolog)
 		{
 			syslog(LOG_WARNING, "flock(LOCK_UN): %s",
 			       strerror(errno));
@@ -485,13 +545,13 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 		struct flock l;
 
 		l.l_start = 0;
-		l.l_end = 0;
+		l.l_len = 0;
 		l.l_type = F_UNLCK;
 		l.l_whence = SEEK_SET;
 
 		status = fcntl(fd, F_SETLKW, &l);
 
-		if (status != 0)
+		if (status != 0 && dolog)
 		{
 			syslog(LOG_WARNING, "fcntl(F_UNLCK): %s",
 			       strerror(errno));
@@ -499,6 +559,8 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 # endif /* LOCK_SH */
 	}
 
+	(void) pthread_mutex_unlock(&pop_lock);
+
 	return ret;
 }
 #endif /* POPAUTH */
@@ -656,3 +718,42 @@ dkimf_stripcr(char *str)
 	if (q != p)
 		*q = *p;
 }
+
+/*
+**  DKIMF_MKPATH -- generate a path
+**
+**  Parameters:
+**  	path -- output buffer
+**  	pathlen -- bytes available at "path"
+**  	root -- root to infer; if empty, use getcwd()
+**  	file -- filename to use
+**
+**  Return value:
+**  	None.
+*/
+
+void
+dkimf_mkpath(char *path, size_t pathlen, char *root, char *file)
+{
+	assert(path != NULL);
+	assert(root != NULL);
+	assert(file != NULL);
+
+	if (file[0] == '/')				/* explicit path */
+	{
+		sm_strlcpy(path, file, pathlen);
+	}
+	else if (root[0] == '\0')			/* no root, use cwd */
+	{
+		(void) getcwd(path, pathlen);
+		sm_strlcat(path, "/", pathlen);
+		sm_strlcat(path, file, pathlen);
+	}
+	else						/* use root */
+	{
+		sm_strlcpy(path, root, pathlen);
+		if (root[strlen(root) - 1] != '/')
+			sm_strlcat(path, "/", pathlen);
+		sm_strlcat(path, file, pathlen);
+	}
+}
