@@ -7,7 +7,7 @@
 #define _DKIM_TYPES_H_
 
 #ifndef lint
-static char dkim_types_h_id[] = "@(#)$Id: dkim-types.h,v 1.38 2007/03/01 18:34:49 msk Exp $";
+static char dkim_types_h_id[] = "@(#)$Id: dkim-types.h,v 1.60 2007/05/21 22:19:33 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -31,6 +31,11 @@ static char dkim_types_h_id[] = "@(#)$Id: dkim-types.h,v 1.38 2007/03/01 18:34:4
 #include <openssl/err.h>
 #include <openssl/sha.h>
 
+#ifdef _FFR_QUERY_CACHE
+/* libdb includes */
+# include <db.h>
+#endif /* _FFR_QUERY_CACHE */
+
 /* libdkim includes */
 #include "dkim.h"
 
@@ -60,6 +65,16 @@ struct dkim_set
 	struct dkim_set *	set_next;
 };
 
+/* struct dkim_siginfo -- signature information for use by the caller */
+struct dkim_siginfo
+{
+	u_int			sig_flags;
+	u_int			sig_index;
+	u_int			sig_bh;
+	void *			sig_context;
+	struct dkim_set *	sig_taglist;
+};
+
 /* struct dkim_sha1 -- stuff needed to do a sha1 hash */
 struct dkim_sha1
 {
@@ -102,6 +117,7 @@ struct dkim
 	bool			dkim_bodydone;
 	bool			dkim_blankline;
 	bool			dkim_subdomain;
+	bool			dkim_skipbody;
 #ifdef _FFR_ALLMAN_SSP_02
 	bool			dkim_userpolicy;
 #endif /* _FFR_ALLMAN_SSP_02 */
@@ -111,13 +127,14 @@ struct dkim
 	int			dkim_timeout;
 	int			dkim_hashtype;
 	int			dkim_sigtype;
-	int			dkim_shcnt;
-#if _FFR_SET_DNS_CALLBACK
-	u_int			dkim_callback_int;
-#endif /* _FFR_SET_DNS_CALLBACK */
+#ifdef _FFR_QUERY_CACHE
+	u_int			dkim_cache_queries;
+	u_int			dkim_cache_hits;
+#endif /* _FFR_QUERY_CACHE */
 	u_int			dkim_blanks;
 	u_int			dkim_keybits;
 	u_int			dkim_version;
+	u_int			dkim_sigcount;
 	size_t			dkim_b64keylen;
 	size_t			dkim_b64siglen;
 	size_t			dkim_keylen;
@@ -131,6 +148,11 @@ struct dkim
 	off_t			dkim_bodylen;
 	off_t			dkim_signlen;
 	off_t			dkim_canonlen;
+#ifdef _FFR_HASH_BUFFERING
+	u_int			dkim_hashbuflen;
+	u_int			dkim_hashbufsize;
+	u_char *		dkim_hashbuf;
+#endif /* _FFR_HASH_BUFFERING */
 	const char *		dkim_id;
 	u_char *		dkim_domain;
 	u_char *		dkim_user;
@@ -138,24 +160,25 @@ struct dkim
 	u_char *		dkim_hdrlist;
 	u_char *		dkim_b64key;
 	u_char *		dkim_b64sig;
-	u_char *		dkim_shlist;
 	u_char *		dkim_key;
 	u_char *		dkim_sig;
 	u_char *		dkim_reportaddr;
 	u_char *		dkim_sender;
 	u_char *		dkim_signer;
 	u_char *		dkim_error;
-	u_char **		dkim_shset;
+	u_char *		dkim_zdecode;
+#ifdef _FFR_VBR
+	u_char *		dkim_vbr_type;
+	u_char *		dkim_vbr_cert;
+	u_char **		dkim_vbr_trusted;
+#endif /* _FFR_VBR */
 	char *			dkim_tmpdir;
 	void *			dkim_hash;
 	void *			dkim_bhash;
 	void *			dkim_signature;
-	void *			dkim_memclosure;
 	void *			dkim_closure;
-#if _FFR_SET_DNS_CALLBACK
-	void *			dkim_user_context;
-	void			(*dkim_dns_callback) (void *context);
-#endif /* _FFR_SET_DNS_CALLBACK */
+	const void *		dkim_user_context;
+	struct dkim_siginfo **	dkim_siglist;
 	struct dkim_set *	dkim_sethead;
 	struct dkim_set *	dkim_settail;
 	struct dkim_set *	dkim_sigset;
@@ -169,19 +192,43 @@ struct dkim
 struct dkim_lib
 {
 	bool			dkiml_signre;
+	bool			dkiml_skipre;
 	u_int			dkiml_flags;
 	u_int			dkiml_timeout;
 	u_int			dkiml_version;
+	u_int			dkiml_callback_int;
+	time_t			dkiml_fixedtime;
+	time_t			dkiml_sigttl;
 	dkim_query_t		dkiml_querymethod;
 	void *			(*dkiml_malloc) (void *closure, size_t nbytes);
 	void			(*dkiml_free) (void *closure, void *p);
-	u_char			dkiml_tmpdir[MAXPATHLEN + 1];
-	u_char			dkiml_queryinfo[MAXPATHLEN + 1];
 #if USE_ARLIB
 	AR_LIB			dkiml_arlib;
 #endif /* USE_ARLIB */
 	u_char **		dkiml_senderhdrs;
+	u_char **		dkiml_alwayshdrs;
+#ifdef _FFR_QUERY_CACHE
+	DB *			dkiml_cache;
+#endif /* _FFR_QUERY_CACHE */
 	regex_t			dkiml_hdrre;
+	regex_t			dkiml_skiphdrre;
+	int			(*dkiml_key_lookup) (DKIM *dkim, u_char *buf,
+				                     size_t buflen);
+	int			(*dkiml_policy_lookup) (DKIM *dkim,
+				                        u_char *buf,
+				                        size_t buflen);
+	void *			(*dkiml_sig_handle) (void *closure);
+	void			(*dkiml_sig_handle_free) (void *closure,
+				                          void *user);
+	void			(*dkiml_sig_tagvalues) (void *user,
+				                        dkim_param_t pcode,
+				                        const u_char *param,
+				                        const u_char *value);
+	DKIM_STAT		(*dkiml_prescreen) (DKIM_SIGINFO **sigs,
+				                    int nsigs);
+	void			(*dkiml_dns_callback) (const void *context);
+	u_char			dkiml_tmpdir[MAXPATHLEN + 1];
+	u_char			dkiml_queryinfo[MAXPATHLEN + 1];
 };
 
 #endif /* _DKIM_TYPES_H_ */
