@@ -7,7 +7,7 @@
 #define _DKIM_H_
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.55 2007/03/02 00:53:59 msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.98 2007/05/23 01:02:50 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -22,6 +22,17 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.55 2007/03/02 00:53:59 msk Exp $"
 /* openssl includes */
 #include <openssl/sha.h>
 
+/*
+**  version -- 0xrrMMmmpp
+**  
+**  	rr == release number
+**  	MM == major revision number
+**  	mm == minor revision number
+**  	pp == patch number
+*/
+
+#define	DKIM_LIB_VERSION	0x01000000
+
 /* the basics */
 #ifndef NULL
 # define NULL	0
@@ -49,6 +60,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.55 2007/03/02 00:53:59 msk Exp $"
 #define MAXHDRCNT		64	/* max. headers signed */
 #define MAXHEADER		4096	/* buffer for caching one header */
 #define MAXHEADERS		32768	/* buffer for caching headers */
+#define MAXLABELS		16	/* max. labels we allow */
 #define MAXPOLICYDEPTH		5	/* policy recursion we allow */
 #define MAXTAGNAME		8	/* biggest tag name */
 
@@ -64,6 +76,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.55 2007/03/02 00:53:59 msk Exp $"
 #define	DKIM_VERSION_SIG_DRAFT2	"0.5"	/* draft version token for sigs */
 
 /* headers */
+#define	DKIM_FROMHEADER		"From"	/* From: header */
 #define	DKIM_SIGNHEADER		"DKIM-Signature"
 					/* DKIM signature header */
 #define	DKIM_HDRMARGIN		75	/* "standard" header margin */
@@ -83,12 +96,13 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.55 2007/03/02 00:53:59 msk Exp $"
 */
 
 #define DKIM_VERSION_UNKNOWN		0
-#define DKIM_VERSION_ALLMAN_BASE_00	1
+#define DKIM_VERSION_UNUSED01		1	/* was ALLMAN_BASE_00 */
 #define DKIM_VERSION_IETF_BASE_00	2
 #define DKIM_VERSION_IETF_BASE_01	3
 #define DKIM_VERSION_IETF_BASE_02	4
 #define DKIM_VERSION_IETF_BASE_10	5
-#define DKIM_VERSION_DEFAULT		DKIM_VERSION_IETF_BASE_10
+#define DKIM_VERSION_RFC4871		6
+#define DKIM_VERSION_DEFAULT		DKIM_VERSION_RFC4871
 
 /*
 **  DKIM_STAT -- status code type
@@ -108,6 +122,17 @@ typedef int DKIM_STAT;
 #define	DKIM_STAT_INVALID	9	/* invalid function parameter */
 #define	DKIM_STAT_NOTIMPLEMENT	10	/* function not implemented */
 #define	DKIM_STAT_KEYFAIL	11	/* key retrieval failed */
+#define	DKIM_STAT_CBREJECT	12	/* callback requested reject */
+#define	DKIM_STAT_CBINVALID	13	/* callback gave invalid result */
+
+/*
+**  DKIM_CBSTAT -- callback status code type
+*/
+
+typedef int DKIM_CBSTAT;
+
+#define	DKIM_CBSTAT_CONTINUE	0	/* continue */
+#define	DKIM_CBSTAT_REJECT	1	/* reject */
 
 /*
 **  DKIM_CANON -- canonicalization method
@@ -115,10 +140,9 @@ typedef int DKIM_STAT;
 
 typedef int dkim_canon_t;
 
-#define DKIM_CANON_UNKNOWN	-1	/* unknown method */
-#define DKIM_CANON_SIMPLE	0	/* as specified in DKIM 01 draft */
-#define DKIM_CANON_RELAXED	1	/* as specified in DKIM 01 draft */
-#define DKIM_CANON_NOWSP	2	/* as specified in DKIM 00 draft */
+#define DKIM_CANON_UNKNOWN	(-1)	/* unknown method */
+#define DKIM_CANON_SIMPLE	0	/* as specified in DKIM spec */
+#define DKIM_CANON_RELAXED	1	/* as specified in DKIM spec */
 
 #define DKIM_CANON_DEFAULT	DKIM_CANON_SIMPLE
 
@@ -128,13 +152,17 @@ typedef int dkim_canon_t;
 
 typedef int dkim_alg_t;
 
-#define DKIM_SIGN_UNKNOWN	-1	/* unknown method */
+#define DKIM_SIGN_UNKNOWN	(-1)	/* unknown method */
 #define DKIM_SIGN_RSASHA1	0	/* an RSA-signed SHA1 digest */
 #ifdef SHA256_DIGEST_LENGTH
 # define DKIM_SIGN_RSASHA256	1	/* an RSA-signed SHA256 digest */
 #endif /* SHA256_DIGEST_LENGTH */
 
-#define DKIM_SIGN_DEFAULT	DKIM_SIGN_RSASHA1
+#ifdef SHA256_DIGEST_LENGTH
+# define DKIM_SIGN_DEFAULT	DKIM_SIGN_RSASHA256
+#else /* SHA256_DIGEST_LENGTH */
+# define DKIM_SIGN_DEFAULT	DKIM_SIGN_RSASHA1
+#endif /* SHA256_DIGEST_LENGTH */
 
 /*
 **  DKIM_QUERY -- query method
@@ -142,11 +170,9 @@ typedef int dkim_alg_t;
 
 typedef int dkim_query_t;
 
-#define DKIM_QUERY_UNKNOWN	-1	/* unknown method */
+#define DKIM_QUERY_UNKNOWN	(-1)	/* unknown method */
 #define DKIM_QUERY_DNS		0	/* DNS query method (per the draft) */
-#if DKIM_QUERY_FILE
-# define DKIM_QUERY_FILE	1	/* text file method */
-#endif /* DKIM_QUERY_FILE */
+#define DKIM_QUERY_FILE		1	/* text file method (for testing) */
 
 #define DKIM_QUERY_DEFAULT	DKIM_QUERY_DNS
 
@@ -156,6 +182,7 @@ typedef int dkim_query_t;
 
 typedef int dkim_param_t;
 
+#define DKIM_PARAM_UNKNOWN	(-1)	/* unknown */
 #define DKIM_PARAM_SIGNATURE	0	/* b */
 #define DKIM_PARAM_SIGNALG	1	/* a */
 #define DKIM_PARAM_DOMAIN	2	/* d */
@@ -246,6 +273,9 @@ typedef int dkim_set_t;
 #define	DKIM_SETTYPE_SIGNATURE	0
 #define	DKIM_SETTYPE_POLICY	1
 #define	DKIM_SETTYPE_KEY	2
+#ifdef _FFR_VBR
+# define DKIM_SETTYPE_VBRINFO	3
+#endif /* _FFR_VBR */
 
 /*
 **  DKIM_HASHTYPE -- types of hashes
@@ -283,11 +313,19 @@ typedef int dkim_opts_t;
 #define	DKIM_OPTS_VERSION	5
 #define	DKIM_OPTS_QUERYMETHOD	6
 #define	DKIM_OPTS_QUERYINFO	7
+#define	DKIM_OPTS_FIXEDTIME	8
+#define	DKIM_OPTS_SKIPHDRS	9
+#define	DKIM_OPTS_ALWAYSHDRS	10
+#define	DKIM_OPTS_SIGNATURETTL	11
 
 #define	DKIM_LIBFLAGS_NONE	0x00
 #define	DKIM_LIBFLAGS_TMPFILES	0x01
 #define	DKIM_LIBFLAGS_KEEPFILES	0x02
 #define	DKIM_LIBFLAGS_SIGNLEN	0x04
+#ifdef _FFR_QUERY_CACHE
+# define DKIM_LIBFLAGS_CACHE	0x10
+#endif /* _FFR_QUERY_CACHE */
+#define DKIM_LIBFLAGS_ZTAGS	0x20
 
 #define	DKIM_LIBFLAGS_DEFAULT	DKIM_LIBFLAGS_NONE
 
@@ -346,12 +384,27 @@ typedef struct dkim_key DKIM_KEY;
 struct dkim_policy;
 typedef struct dkim_policy DKIM_POLICY;
 
+/*
+**  DKIM_SIGINFO -- signature information for use by the caller
+*/
+
+struct dkim_siginfo;
+typedef struct dkim_siginfo DKIM_SIGINFO;
+
+#define DKIM_SIGFLAG_IGNORE	0x01
+#define DKIM_SIGFLAG_PROCESSED	0x02
+#define DKIM_SIGFLAG_PASSED	0x04
+
+#define DKIM_SIGBH_UNTESTED	(-1)
+#define DKIM_SIGBH_MATCH	0
+#define DKIM_SIGBH_MISMATCH	1
+
 /*
 **  PROTOTYPES
 */
 
 /*
-**  dkim_init -- initialize the DKIM package
+**  DKIM_INIT -- initialize the DKIM package
 **
 **  Parameters:
 **  	None.
@@ -364,7 +417,19 @@ extern DKIM_LIB *dkim_init __P((void *(*mallocf)(void *closure, size_t nbytes),
                                 void (*freef)(void *closure, void *p)));
 
 /*
-**  dkim_sign -- make a new DKIM context for signing
+**  DKIM_CLOSE -- shut down the DKIM package
+**
+**  Parameters:
+**  	lib -- DKIM_LIB handle to shut down
+**
+**  Return value:
+**  	None.
+*/
+
+extern void dkim_close __P((DKIM_LIB *lib));
+
+/*
+**  DKIM_SIGN -- make a new DKIM context for signing
 **
 **  Parameters:
 **  	libhandle -- library handle, returned by dkim_init()
@@ -398,7 +463,7 @@ extern DKIM *dkim_sign __P((DKIM_LIB *libhandle, const char *id,
                             off_t length, DKIM_STAT *statp));
 
 /*
-**  dkim_verify -- make a new DKIM context for verifying
+**  DKIM_VERIFY -- make a new DKIM context for verifying
 **
 **  Parameters:
 **  	libhandle -- library handle, returned by dkim_init()
@@ -416,7 +481,7 @@ extern DKIM *dkim_verify __P((DKIM_LIB *libhandle, const char *id,
                               void *memclosure, DKIM_STAT *statp));
 
 /*
-**  dkim_header -- process a header
+**  DKIM_HEADER -- process a header
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_sign() or
@@ -431,7 +496,7 @@ extern DKIM *dkim_verify __P((DKIM_LIB *libhandle, const char *id,
 extern DKIM_STAT dkim_header __P((DKIM *dkim, u_char *hdr, size_t len));
 
 /*
-**  dkim_eoh -- identify end of headers
+**  DKIM_EOH -- identify end of headers
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_sign() or
@@ -445,7 +510,7 @@ extern DKIM_STAT dkim_header __P((DKIM *dkim, u_char *hdr, size_t len));
 extern DKIM_STAT dkim_eoh __P((DKIM *dkim));
 
 /*
-**  dkim_body -- process a body chunk
+**  DKIM_BODY -- process a body chunk
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_sign() or
@@ -460,7 +525,7 @@ extern DKIM_STAT dkim_eoh __P((DKIM *dkim));
 extern DKIM_STAT dkim_body __P((DKIM *dkim, u_char *buf, size_t len));
 
 /*
-**  dkim_eom -- identify end of body
+**  DKIM_EOM -- identify end of body
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_sign() or
@@ -475,7 +540,7 @@ extern DKIM_STAT dkim_body __P((DKIM *dkim, u_char *buf, size_t len));
 extern DKIM_STAT dkim_eom __P((DKIM *dkim, bool *testkey));
 
 /*
-**  dkim_getsighdr -- compute and return a signature header for a message
+**  DKIM_GETSIGHDR -- compute and return a signature header for a message
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_sign()
@@ -492,7 +557,7 @@ extern DKIM_STAT dkim_getsighdr __P((DKIM *dkim, u_char *buf, size_t len,
                                      size_t margin, size_t initial));
 
 /*
-**  dkim_getcanonlen -- report number of (canonicalized) body bytes that were
+**  DKIM_GETCANONLEN -- report number of (canonicalized) body bytes that were
 **                      signed
 **
 **  Parameters:
@@ -513,7 +578,7 @@ extern DKIM_STAT dkim_getcanonlen __P((DKIM *dkim, off_t *msglen,
                                        off_t *canonlen));
 
 /*
-**  dkim_reportinfo -- return info needed to generate failure report
+**  DKIM_REPORTINFO -- return info needed to generate failure report
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_verify()
@@ -546,27 +611,6 @@ extern DKIM_STAT dkim_reportinfo __P((DKIM *dkim, int *fd, int *bfd,
 extern DKIM_STAT dkim_options __P((DKIM_LIB *dklib, int op, dkim_opts_t opt,
                                    void *ptr, size_t len));
 
-/*
-**  dkim_getidentity -- retrieve signer's apparent identity
-**
-**  Parameters:
-**  	dkim -- a DKIM handle previously returned by dkim_sign() or
-**  	        dkim_verify()
-**  	hname -- buffer to receive name of the header from which identity
-**  	         was extracted
-**  	hnamelen -- number of bytes available at hname
-**  	hval -- buffer to receive decommented value of the header from which
-**  	        identity was extracted
-**  	hvallen -- number of bytes available at hval
-**
-**  Return value:
-**  	A DKIM_STAT value.
-*/
-
-extern DKIM_STAT dkim_getidentity __P((DKIM *dkim,
-                                       u_char *hname, size_t hnamelen,
-                                       u_char *hval, size_t hvallen));
-
 /*
 **  DKIM_POLICY -- retreive signing policy associated with the sender's domain
 **
@@ -619,28 +663,222 @@ extern DKIM_STAT dkim_getsignalg __P((DKIM *dkim, dkim_alg_t *alg));
 
 extern DKIM_STAT dkim_getsigntime __P((DKIM *dkim, time_t *when));
 
-#if _FFR_SET_DNS_CALLBACK
+/*
+**  DKIM_GETSELECTOR -- retrieve selector used to generate the signature
+**
+**  Parameters:
+**  	dkim -- DKIM handle from which to retrieve selector
+**
+**  Return value:
+**  	Selector found in the signature.
+*/
+
+extern const char * dkim_getselector __P((DKIM *dkim));
+
+/*
+**  DKIM_GETSIGDOMAIN -- retrieve signing domain after verifying
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**
+**  Return value:
+**  	Pointer to the signing domain.
+*/
+
+extern const char *dkim_getsigdomain __P((DKIM *dkim));
+
+#ifdef _FFR_STATS
+/*
+**  DKIM_GETCANONS -- retrieve canonicaliztions after verifying
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**
+**  Return value:
+**  	Pointer to the signing domain.
+*/
+
+extern DKIM_STAT dkim_getcanons __P((DKIM *dkim, dkim_canon_t *hdr,
+                                     dkim_canon_t *body));
+
+/*
+**  DKIM_GETVERSION -- retreive signing version after verifying
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	when -- timestamp on the signature (returned)
+**
+**  Return value:
+**  	A DKIM_STAT value.
+*/
+
+extern DKIM_STAT dkim_getversion __P((DKIM *dkim, int *ver));
+#endif /* _FFR_STATS */
+
+/*
+**  DKIM_SET_USER_CONTEXT -- set DKIM handle user context
+**
+**  Parameters:
+**  	dkim -- DKIM signing handle
+**  	ctx -- user context pointer to store
+**
+**  Parameters:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_set_user_context __P((DKIM *dkim, const void *ctx));
+
+/*
+**  DKIM_SET_SIGNER -- set DKIM signature's signer
+**
+**  Parameters:
+**  	dkim -- DKIM signing handle
+**  	signer -- signer to store
+**
+**  Parameters:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_set_signer __P((DKIM *dkim, const char *signer));
+
 /*
 **  DKIM_SET_DNS_CALLBACK -- set the DNS wait callback
 **
 **  Parameters:
-**  	dkim -- DKIM handle
+**  	libdkim -- DKIM library handle
 **  	func -- function to call; should take an opaque context pointer
-**  	ctx -- opaque context handle
 **  	interval -- how often to call back
 **
 **  Return value:
 **  	None.
 */
 
-extern DKIM_STAT dkim_set_dns_callback __P((DKIM *dkim,
-                                            void (*func)(void *context),
-                                            void *ctx,
+extern DKIM_STAT dkim_set_dns_callback __P((DKIM_LIB *libdkim,
+                                            void (*func)(const void *context),
                                             unsigned int interval));
-#endif /* _FFR_SET_DNS_CALLBACK */
 
 /*
-**  dkim_free -- release resources associated with a DKIM handle
+**  DKIM_SET_KEY_LOOKUP -- set the key lookup function
+**
+**  Parameters:
+**  	libdkim -- DKIM library handle
+**  	func -- function to call
+**
+**  Return value:
+**  	DKIM_STAT_OK
+*/
+
+extern DKIM_STAT dkim_set_key_lookup __P((DKIM_LIB *libdkim,
+                                          DKIM_STAT (*func)(DKIM *dkim,
+                                                            u_char *buf,
+                                                            size_t buflen)));
+
+/*
+**  DKIM_SET_POLICY_LOOKUP -- set the policy lookup function
+**
+**  Parameters:
+**  	libdkim -- DKIM library handle
+**  	func -- function to call
+**
+**  Return value:
+**  	DKIM_STAT_OK
+*/
+
+extern DKIM_STAT dkim_set_policy_lookup __P((DKIM_LIB *libdkim,
+                                             int (*func)(DKIM *dkim,
+                                                         u_char *buf,
+                                                         size_t buflen)));
+
+/*
+**  DKIM_SET_SIGNATURE_HANDLE -- set the signature handle creator function
+**
+**  Parameters:
+**  	libdkim -- DKIM library handle
+**  	func -- function to call
+**
+**  Return value:
+**  	Pointer to the user-side handle thus created, or NULL.
+*/
+
+extern DKIM_STAT dkim_set_signature_handle __P((DKIM_LIB *libdkim,
+                                                void * (*func)(void *closure)));
+
+/*
+**  DKIM_SET_SIGNATURE_HANDLE_FREE -- set the signature handle destroyer
+**                                    function
+**
+**  Parameters:
+**  	libdkim -- DKIM library handle
+**  	func -- function to call
+**
+**  Return value:
+**  	None.
+*/
+
+extern DKIM_STAT dkim_set_signature_handle_free __P((DKIM_LIB *libdkim,
+                                                     void (*func)(void *closure,
+                                                                  void *user)));
+
+/*
+**  DKIM_SET_SIGNATURE_TAGVALUES -- set the signature handle populator function
+**
+**  Parameters:
+**  	libdkim -- DKIM library handle
+**  	func -- function to call
+**
+**  Return value:
+**  	DKIM_STAT_OK
+*/
+
+extern DKIM_STAT dkim_set_signature_tagvalues __P((DKIM_LIB *libdkim,
+                                                   void (*func)(void *user,
+                                                                dkim_param_t pcode,
+                                                                const u_char *param,
+                                                                const u_char *value)));
+
+/*
+**  DKIM_SET_PRESCREEN -- set the prescreen function
+**
+**  Parameters:
+**  	libdkim -- DKIM library handle
+**  	func -- function to call
+**
+**  Return value:
+**  	DKIM_STAT_OK
+*/
+
+extern DKIM_STAT dkim_set_prescreen __P((DKIM_LIB *libdkim,
+                                         DKIM_CBSTAT (*func)(DKIM_SIGINFO **sigs,
+                                                             int nsigs)));
+
+/*
+**  DKIM_SIG_GETCONTEXT -- get user-specific context from a DKIM_SIGINFO
+**
+**  Parameters:
+**  	siginfo -- a pointer to a DKIM_SIGINFO
+**
+**  Return value:
+**  	The user-provided pointer stored in the named "siginfo", or NULL
+**  	if none was ever set.
+*/
+
+extern void *dkim_sig_getcontext __P((DKIM_SIGINFO *siginfo));
+
+/*
+**  DKIM_SIG_IGNORE -- mark a signature referenced by a DKIM_SIGINFO with
+**                     an "ignore" flag
+**
+**  Parameters:
+**  	siginfo -- pointer to a DKIM_SIGINFO to update
+**
+**  Return value:
+**  	None.
+*/
+
+extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
+
+/*
+**  DKIM_FREE -- release resources associated with a DKIM handle
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_sign() or
@@ -666,7 +904,21 @@ extern DKIM_STAT dkim_free __P((DKIM *dkim));
 extern const char *dkim_geterror __P((DKIM *dkim));
 
 /*
-**  rfc2822_mailbox_split -- extract the userid and host from a structured
+**  DKIM_OHDRS -- extract and decode original headers
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	ptrs -- user-provided array of pointers to header strings (updated)
+**  	pcnt -- number of pointers available (updated)
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_ohdrs __P((DKIM *dkim, char **ptrs, int *pcnt));
+
+/*
+**  RFC2822_MAILBOX_SPLIT -- extract the userid and host from a structured
 **                           header
 **
 **  Parameters:
@@ -680,4 +932,42 @@ extern const char *dkim_geterror __P((DKIM *dkim));
 
 extern int rfc2822_mailbox_split __P((char *addr, char **user, char **domain));
 
+/*
+**  DKIM_SSL_VERSION -- return the version of the OpenSSL library against
+**                      which this library was compiled
+**
+**  Parameters:
+**  	None.
+**
+**  Return value:
+**  	The OPENSSL_VERSION_NUMBER constant as defined by OpenSSL.
+*/
+
+extern unsigned long dkim_ssl_version __P((void));
+
+#ifdef _FFR_QUERY_CACHE
+/*
+**  DKIM_FLUSH_CACHE -- purge expired records from the database, reclaiming
+**                      space for use by new data
+**
+**  Parameters:
+**  	lib -- DKIM library whose cache should be flushed
+**
+**  Return value:
+**  	-1 -- caching is not in effect
+**  	>= 0 -- number of flushed records
+*/
+
+extern int dkim_flush_cache __P((DKIM_LIB *lib));
+#endif /* _FFR_QUERY_CACHE */
+
+/* default list of sender headers */
+extern const u_char *default_senderhdrs[];
+
+/* default list of headers to sign */
+extern const u_char *default_signhdrs[];
+
+/* default list of headers to skip */
+extern const u_char *default_skiphdrs[];
+
 #endif /* ! _DKIM_H_ */
